@@ -146,6 +146,7 @@ qw(
 	is_cross_compiling
 	is_build_profile_active
 	get_buildoption
+	perl_cross_incdir
 ),
 	# Other
 qw(
@@ -166,6 +167,7 @@ qw(
 	DEFAULT_PACKAGE_TYPE
 	DBGSYM_PACKAGE_TYPE
 	DH_BUILTIN_VERSION
+	is_known_package
 	assert_opt_is_known_package
 	restore_all_files
 
@@ -184,6 +186,7 @@ our $PKGVERSION_REGEX = qr/
                  [0-9][0-9A-Za-z.+:~]*       # Upstream version (with no hyphens)
                  (?: - [0-9A-Za-z.+:~]+ )*   # Optional debian revision (+ upstreams versions with hyphens)
                           /xoa;
+our $MAINTSCRIPT_TOKEN_REGEX = qr/[A-Za-z0-9_.+]+/o;
 
 # From Policy 5.1:
 #
@@ -197,9 +200,21 @@ our $DEB822_FIELD_REGEX = qr/
 	    [\x21-\x39\x3B-\x7F]*                  # Subsequent characters (if any)
     /xoa;
 
+our $PARSE_DH_SEQUENCE_INFO = 0;
+
+# We need logging in compat 9 or in override/hook targets (for --remaining-packages to work)
+# - This option is a global toggle to disable logs for special commands (e.g. dh or dh_clean)
+# It is initialized during "init".  This implies that commands that never calls init are
+# not dh_* commands or do not need the log
+my $write_log = undef;
+
 sub init {
 	my %params=@_;
 
+	if ($params{internal_parse_dh_sequence_info}) {
+		$PARSE_DH_SEQUENCE_INFO = 1;
+	}
+
 	# Check if we can by-pass the expensive Getopt::Long by optimising for the
 	# common case of "-a" or "-i"
 	if (scalar(@ARGV) == 1 && ($ARGV[0] eq '-a' || $ARGV[0] eq '-i') &&
@@ -300,17 +315,25 @@ sub init {
 	}
 
 	$dh{U_PARAMS} //= [];
+
+	if ($params{'inhibit_log'}) {
+		$write_log = 0;
+	} else {
+		# Only initialize if unset (i.e. avoid overriding an early call
+		# to inhibit_log()
+		$write_log //= 1;
+	}
 }
 
-# Run at exit. Add the command to the log files for the packages it acted
-# on, if it's exiting successfully.
-my $write_log=1;
+# Ensure the log is written if requested but only if the command was
+# successful.
 sub END {
+	return if $? != 0 or not $write_log;
 	# If there is no 'debian/control', then we are not being run from
 	# a package directory and then the write_log will not do what we
 	# expect.
 	return if not -f 'debian/control';
-	if ($? == 0 && $write_log && (compat(9, 1) || $ENV{DH_INTERNAL_OVERRIDE})) {
+	if (compat(9, 1) || $ENV{DH_INTERNAL_OVERRIDE}) {
 		write_log(basename($0), @{$dh{DOPACKAGES}});
 	}
 }
@@ -321,20 +344,6 @@ sub logfile {
 	return "debian/${ext}debhelper.log"
 }
 
-sub add_override {
-	my $line=shift;
-	$line="override_$ENV{DH_INTERNAL_OVERRIDE} $line"
-		if defined $ENV{DH_INTERNAL_OVERRIDE};
-	return $line;
-}
-
-sub remove_override {
-	my $line=shift;
-	$line=~s/^\Qoverride_$ENV{DH_INTERNAL_OVERRIDE}\E\s+//
-		if defined $ENV{DH_INTERNAL_OVERRIDE};
-	return $line;
-}
-
 sub load_log {
 	my ($package, $db)=@_;
 
@@ -342,7 +351,7 @@ sub load_log {
 	open(LOG, "<", logfile($package)) || return;
 	while (<LOG>) {
 		chomp;
-		my $command=remove_override($_);
+		my $command = $_;
 		push @log, $command;
 		$db->{$package}{$command}=1 if defined $db;
 	}
@@ -357,9 +366,9 @@ sub write_log {
 	return if $dh{NO_ACT};
 
 	foreach my $package (@packages) {
-		my $log=logfile($package);
+		my $log = logfile($package);
 		open(LOG, ">>", $log) || error("failed to write to ${log}: $!");
-		print LOG add_override($cmd)."\n";
+		print LOG $cmd."\n";
 		close LOG;
 	}
 }
@@ -370,8 +379,8 @@ sub commit_override_log {
 	return if $dh{NO_ACT};
 
 	foreach my $package (@packages) {
-		my @log=map { remove_override($_) } load_log($package);
-		my $log=logfile($package);
+		my @log = load_log($package);
+		my $log = logfile($package);
 		open(LOG, ">", $log) || error("failed to write to ${log}: $!");
 		print LOG $_."\n" foreach @log;
 		close LOG;
@@ -453,7 +462,7 @@ sub _doit {
 		if (defined($options)) {
 			if (defined(my $dir = $options->{chdir})) {
 				if ($dir ne '.') {
-					chdir($dir) or error("chdir(\"${dir}\) failed: $!");
+					chdir($dir) or error("chdir(\"${dir}\") failed: $!");
 				}
 			}
 			open(STDIN, '<', '/dev/null') or error("redirect STDIN failed: $!");
@@ -758,19 +767,57 @@ sub nonquiet_print {
 	}
 }
 
-# Output an error message and die (can be caught).
-sub error {
-	my $message=shift;
+{
+	my $_use_color;
+	sub _color {
+		my ($msg, $color) = @_;
+		if (not defined($_use_color)) {
+			# This part is basically Dpkg::ErrorHandling::setup_color over again
+			# with some tweaks.
+			# (but the module uses Dpkg + Dpkg::Gettext, so it is very expensive
+			# to load)
+			my $mode = $ENV{'DH_COLORS'} // $ENV{'DPKG_COLORS'};
+			# Support NO_COLOR (https://no-color.org/)
+			$mode //= exists($ENV{'NO_COLOR'}) ? 'never' : 'auto';
 
-	die basename($0).": $message\n";
-}
+			if ($mode eq 'auto') {
+				$_use_color = 1 if -t *STDOUT or -t *STDERR;
+			} elsif ($mode eq 'always') {
+				$_use_color = 1;
+			} else {
+				$_use_color = 0;
+			}
 
-# Output a warning.
-sub warning {
-	my ($message) = @_;
-	$message //= '';
-	
-	print STDERR basename($0).": $message\n";
+			eval {
+				require Term::ANSIColor if $_use_color;
+			};
+			if ($@) {
+				# In case of errors, skip colors.
+				$_use_color = 0;
+			}
+		}
+		if ($_use_color) {
+			local $ENV{'NO_COLOR'} = undef;
+			$msg = Term::ANSIColor::colored($msg, $color);
+		}
+		return $msg;
+	}
+
+	# Output an error message and die (can be caught).
+	sub error {
+		my ($message) = @_;
+		# ensure the error code is well defined.
+		$! = 255;
+		die(_color(basename($0), 'bold') . ': ' . _color('error', 'bold red') . ": $message\n");
+	}
+
+	# Output a warning.
+	sub warning {
+		my ($message) = @_;
+		$message //= '';
+
+		print STDERR _color(basename($0), 'bold') . ': ' . _color('warning', 'bold yellow') . ": $message\n";
+	}
 }
 
 # Returns the basename of the argument passed to it.
@@ -832,6 +879,9 @@ my $compat_from_bd;
 						warning("Please specify the debhelper compat level exactly once.");
 						warning(" * debian/compat requests compat ${new_compat}.");
 						warning(" * debian/control requests compat ${compat_from_bd} via \"debhelper-compat (= ${compat_from_bd})\"");
+						warning();
+						warning("Hint: If you just added a build-dependency on debhelper-compat, then please remember to remove debian/compat");
+						warning();
 						error("debhelper compat level specified both in debian/compat and via build-dependency on debhelper-compat");
 					}
 					$c = $new_compat;
@@ -965,11 +1015,7 @@ sub default_sourcedir {
 			}
 		}
 		foreach my $file (@try) {
-			if (-f $file &&
-				(! $dh{IGNORE} || ! exists $dh{IGNORE}->{$file})) {
-				return $file;
-			}
-
+			return $file if -f $file;
 		}
 
 		return "";
@@ -1158,7 +1204,7 @@ sub autoscript_sed {
 	my ($sed, $infile, $outfile, $out_fd) = @_;
 	if (not defined($sed) or ref($sed)) {
 		my $out = $out_fd;
-		open(my $in, '<', $infile) or die "$infile: $!";
+		open(my $in, '<', $infile) or error("open $infile failed: $!");
 		if (not defined($out_fd)) {
 			open($out, '>>', $outfile) or error("open($outfile): $!");
 		}
@@ -1173,9 +1219,9 @@ sub autoscript_sed {
 			}
 		}
 		if (not defined($out_fd)) {
-			close($out) or error("close($outfile): $!");
+			close($out) or error("close $outfile failed: $!");
 		}
-		close($in) or die "$infile: $!";
+		close($in) or error("close $infile failed: $!");
 	}
 	else {
 		error("Internal error - passed open handle for legacy method") if defined($out_fd);
@@ -1363,25 +1409,91 @@ sub glob_expand {
 	return @result;
 }
 
+
+my %BUILT_IN_SUBST = (
+	'Space'        => ' ',
+	'Dollar'       => '$',
+	'Newline'      => "\n",
+	'Tab'          => "\b",
+);
+
+sub _variable_substitution {
+	my ($text, $loc) = @_;
+	return $text if index($text, '$') < 0;
+	my $pos = -1;
+	my $subst_count = 0;
+	my $expansion_count = 0;
+	my $current_size = length($text);
+	my $expansion_size_limit = 3 * $current_size;
+	1 while ($text =~ s<
+			\$\{([A-Za-z0-9][-_:0-9A-Za-z]*)\}  # Match ${something} and replace it
+		>[
+			my $match = $1;
+			my $new_pos = pos()//-1;
+			my $value;
+
+			if ($pos == $new_pos) {
+				# Safe-guard in case we ever implement recursive expansion
+				error("Error substituting in ${loc} (at position $pos); recursion limit while expanding \${${match}")
+					if (++$subst_count >= 20);
+			} else {
+				$subst_count = 0;
+				$pos = $new_pos;
+				if (++$expansion_count >= 50) {
+					error("Error substituting in ${loc}; substitution limit of ${expansion_count} reached");
+				}
+			}
+			if (exists($BUILT_IN_SUBST{$match})) {
+				$value = $BUILT_IN_SUBST{$match};
+			} elsif ($match =~ m/^DEB_(?:BUILD|HOST|TARGET)_/) {
+				$value = dpkg_architecture_value($match) //
+					error(qq{Cannot expand "\${${match}}\" in ${loc} as it is not a known dpkg-architecture value});
+			} elsif ($match =~ m/^env:(.+)/) {
+				my $env_var = $1;
+				$value = $ENV{$env_var} //
+					error(qq{Cannot expand "\${${match}}" in ${loc} as the ENV variable "${env_var}" is unset});
+			}
+			error("Cannot resolve variable \${$match} in ${loc}")
+				if not defined($value);
+			# We do not support recursive expansion.
+			$value =~ s/\$/\$\{\}/;
+			$current_size += length($value) - length($match) - 3;
+			if ($current_size > 4096 and $current_size > $expansion_size_limit) {
+				error("Refusing to expand \${${match}} in ${loc} - the original input seems to grow beyond reasonable'
+						 . ' limits!");
+			}
+			$value;
+		]gex);
+	$text =~ s/\$\{\}/\$/g;
+
+	return $text;
+}
+
 # Reads in the specified file, one line at a time. splits on words, 
 # and returns an array of arrays of the contents.
 # If a value is passed in as the second parameter, then glob
 # expansion is done in the directory specified by the parameter ("." is
 # frequently a good choice).
+# In compat 13+, it will do variable expansion (after splitting the lines
+# into words)
 sub filedoublearray {
 	my ($file, $globdir, $error_handler) = @_;
 
 	# executable config files are a v9 thing.
 	my $x=! compat(8) && -x $file;
+	my $expand_patterns = compat(12) ? 0 : 1;
+	my $source;
 	if ($x) {
 		require Cwd;
 		my $cmd=Cwd::abs_path($file);
 		$ENV{"DH_CONFIG_ACT_ON_PACKAGES"} = join(",", @{$dh{"DOPACKAGES"}});
 		open(DH_FARRAY_IN, '-|', $cmd) || error("cannot run $file: $!");
 		delete $ENV{"DH_CONFIG_ACT_ON_PACKAGES"};
+		$source = "output of ./${file}";
 	}
 	else {
 		open (DH_FARRAY_IN, '<', $file) || error("cannot read $file: $!");
+		$source = $file;
 	}
 
 	my @ret;
@@ -1399,10 +1511,14 @@ sub filedoublearray {
 		# We always ignore/permit empty lines
 		next if $_ eq '';
 		my @line;
+		my $source_ref = "${source} (line $.)";
 
 		if (defined($globdir) && ! $x) {
 			if (ref($globdir)) {
 				my @patterns = split;
+				if ($expand_patterns) {
+					@patterns = map {_variable_substitution($_, $source_ref)} @patterns;
+				}
 				push(@line, glob_expand($globdir, $error_handler, @patterns));
 			} else {
 				# Legacy call - Silently discards globs that match nothing.
@@ -1412,12 +1528,18 @@ sub filedoublearray {
 				# filenames that come out are relative to it.
 				foreach (map { glob "$globdir/$_" } split) {
 					s#^$globdir/##;
+					if ($expand_patterns) {
+						$_ = _variable_substitution($_, $source_ref);
+					}
 					push @line, $_;
 				}
 			}
 		}
 		else {
 			@line = split;
+			if ($expand_patterns) {
+				@line = map {_variable_substitution($_, $source_ref)} @line;
+			}
 		}
 		push @ret, [@line];
 	}
@@ -1570,18 +1692,20 @@ sub getpackages {
 
 	$type //= 'all-listed-in-control-file';
 
-	if (%packages_by_type) {
-		return @{$packages_by_type{$type}};
+	if (not %packages_by_type) {
+		_parse_debian_control();
 	}
+	return @{$packages_by_type{$type}};
+}
 
-	my $fd;
+sub _parse_debian_control {
 	my $package="";
 	my $arch="";
 	my $section="";
 	my $valid_pkg_re = qr{^${PKGNAME_REGEX}$}o;
 	my ($package_type, $multiarch, %seen, @profiles, $source_section,
 		$included_in_build_profile, $cross_type, $cross_target_arch,
-		%bd_fields, $bd_field_value, %seen_fields);
+		%bd_fields, $bd_field_value, %seen_fields, $fd);
 	if (exists $ENV{'DEB_BUILD_PROFILES'}) {
 		@profiles=split /\s+/, $ENV{'DEB_BUILD_PROFILES'};
 	}
@@ -1642,6 +1766,11 @@ sub getpackages {
 	error("could not find Source: line in control file.") if not defined($sourcepackage);
 	if (%bd_fields) {
 		my ($dh_compat_bd, $final_level);
+		my %field2addon_type = (
+			'build-depends' => 'both',
+			'build-depends-arch' => 'arch',
+			'build-depends-indep' => 'indep',
+		);
 		for my $field (sort(keys(%bd_fields))) {
 			my $value = join(' ', @{$bd_fields{$field}});
 			$value =~ s/^\s*//;
@@ -1681,14 +1810,26 @@ sub getpackages {
 					error("Could not parse desired debhelper compat level from relation: $dep");
 				}
 				# Build-Depends on dh-sequence-<foo> OR dh-sequence-<foo> (<op> <version>)
-				if ($dep =~ m/^dh-sequence-(${PKGNAME_REGEX})\s*(?:[(]\s*(?:[<>]?=|<<|>>)\s*(${PKGVERSION_REGEX})\s*[)])?$/) {
+				if ($PARSE_DH_SEQUENCE_INFO and $dep =~ m/^dh-sequence-(${PKGNAME_REGEX})\s*(?:[(]\s*(?:[<>]?=|<<|>>)\s*(?:${PKGVERSION_REGEX})\s*[)])?(\s*[^\|]+[]>]\s*)?$/) {
 					my $sequence = $1;
-					if ($field ne 'build-depends') {
-						warning("Ignoring dh sequence add-on request for sequenece ${sequence} via ${field}: Please move it to the Build-Depends field");
-						warning("The relation that triggered this warning was: ${dep} (from the ${field} field)");
-						next;
+					my $has_profile_or_arch_restriction = $2 ? 1 : 0;
+					my $addon_type = $field2addon_type{$field};
+					if (not defined($field)) {
+						warning("Cannot map ${field} to an add-on type (like \"both\", \"indep\" or \"arch\")");
+						error("Internal error: Cannot satisfy dh sequence add-on request for sequence ${sequence} via ${field}.");
 					}
-					$dh_bd_sequences{$sequence} = 1;
+					if (defined($dh_bd_sequences{$sequence})) {
+						error("Saw $dep multiple times (last time in $field).  However dh only support that build-"
+							. 'dependency at most once across all Build-Depends(-Arch|-Indep) fields');
+					}
+					if ($has_profile_or_arch_restriction) {
+						require Dpkg::Deps;
+						my $dpkg_dep = Dpkg::Deps::deps_parse($dep, build_profiles => \@profiles, build_dep => 1,
+							reduce_restrictions => 1);
+						# If dpkg reduces it to nothing, then it was not relevant for us after all
+						next if not $dpkg_dep;
+					}
+					$dh_bd_sequences{$sequence} = $addon_type;
 				}
 			}
 		}
@@ -1828,8 +1969,6 @@ sub getpackages {
 		}
 	}
 	close($fd);
-
-	return @{$packages_by_type{$type}};
 }
 
 # Return true if we should use root.
@@ -1981,7 +2120,7 @@ sub bd_dh_sequences {
 	# Use $sourcepackage as check because %dh_bd_sequence can be empty
 	# after running getpackages().
 	getpackages() if not defined($sourcepackage);
-	return sort(keys(%dh_bd_sequences));
+	return \%dh_bd_sequences;
 }
 
 sub _concat_slurp_script_files {
@@ -1996,43 +2135,67 @@ sub _concat_slurp_script_files {
 	return $res;
 }
 
+sub _substitution_generator {
+	my ($input) = @_;
+	my $cache = {};
+	return sub {
+		my ($orig_key) = @_;
+		return $cache->{$orig_key} if exists($cache->{$orig_key});
+		my $value = exists($input->{$orig_key}) ? $input->{$orig_key} : undef;
+		if (not defined($value)) {
+			if ($orig_key =~ m/^DEB_(?:BUILD|HOST|TARGET)_/) {
+				$value = dpkg_architecture_value($orig_key);
+			} elsif ($orig_key =~ m{^ENV[.](\S+)$}) {
+				$value = $ENV{$1} // '';
+			}
+		} elsif (ref($value) eq 'CODE') {
+			$value = $value->($orig_key);
+		} elsif ($value =~ s/^@//) {
+			$value = _concat_slurp_script_files($value);
+		}
+		$cache->{$orig_key} = $value;
+		return $value;
+	};
+}
+
 # Handles #DEBHELPER# substitution in a script; also can generate a new
 # script from scratch if none exists but there is a .debhelper file for it.
 sub debhelper_script_subst {
-	my $package=shift;
-	my $script=shift;
-	
+	my ($package, $script, $extra_vars) = @_;
+
 	my $tmp=tmpdir($package);
 	my $ext=pkgext($package);
 	my $file=pkgfile($package,$script);
+	my %variables = defined($extra_vars) ? %{$extra_vars} : ();
 	my $service_script = generated_file($package, "${script}.service", 0);
 	my @generated_scripts = ("debian/$ext$script.debhelper", $service_script);
+	my $subst;
+	@generated_scripts = grep { -f } @generated_scripts;
 	if ($script eq 'prerm' or $script eq 'postrm') {
 		@generated_scripts = reverse(@generated_scripts);
 	}
-	@generated_scripts = grep { -f } @generated_scripts;
+	if (not exists($variables{'DEBHELPER'})) {
+		$variables{'DEBHELPER'} = sub {
+			return _concat_slurp_script_files(@generated_scripts);
+		};
+	}
+	$subst = _substitution_generator(\%variables);
 
 	if ($file ne '') {
-		if (@generated_scripts) {
-			if ($dh{VERBOSE}) {
-				verbose_print('cp -f ' . escape_shell($file) . " $tmp/DEBIAN/$script");
-				verbose_print("perl -p -i -e \"s~#DEBHELPER#~qx{cat @generated_scripts}~eg\" $tmp/DEBIAN/$script");
+		if ($dh{VERBOSE}) {
+			verbose_print('cp -f ' . escape_shell($file) . " $tmp/DEBIAN/$script");
+			verbose_print("[META] Replace #TOKEN#s in \"$tmp/DEBIAN/$script\"");
+		}
+		if (not $dh{NO_ACT}) {
+			my $regex = qr{#(${MAINTSCRIPT_TOKEN_REGEX})#}o;
+			open(my $out_fd, '>', "$tmp/DEBIAN/$script") or error("open($tmp/DEBIAN/$script) failed: $!");
+			open(my $in_fd, '<', $file) or error("open($file) failed: $!");
+			while (my $line = <$in_fd>) {
+				$line =~ s{$regex}{$subst->($1) // "#${1}#"}ge;
+				print {$out_fd} $line;
 			}
-			# Add this into the script, where it has #DEBHELPER#
-			my $text = _concat_slurp_script_files(@generated_scripts);
-			if (not $dh{NO_ACT}) {
-				open(my $out_fd, '>', "$tmp/DEBIAN/$script") or error("open($tmp/DEBIAN/$script) failed: $!");
-				open(my $in_fd, '<', $file) or error("open($file) failed: $!");
-				while (my $line = <$in_fd>) {
-					$line =~ s/#DEBHELPER#/$text/g;
-					print {$out_fd} $line;
-				}
-				close($in_fd);
-				close($out_fd) or error("close($tmp/DEBIAN/$script) failed: $!");
-			}
-		} else {
-			# Just get rid of any #DEBHELPER# in the script.
-			doit({ stdout => "$tmp/DEBIAN/$script" }, 'sed', 's/#DEBHELPER#//', $file);
+			close($in_fd);
+			close($out_fd) or error("close($tmp/DEBIAN/$script) failed: $!");
 		}
 		reset_perm_and_owner('0755', "$tmp/DEBIAN/$script");
 	}
@@ -2255,6 +2418,42 @@ sub get_source_date_epoch {
 	return $ENV{SOURCE_DATE_EPOCH} = $timestamp->epoch();
 }
 
+# Setup the build ENV by setting dpkg-buildflags (via set_buildflags()) plus
+# cleaning up HOME (etc) in compat 13+
+sub setup_buildenv {
+	set_buildflags();
+	if (not compat(12)) {
+		setup_home_and_xdg_dirs();
+	}
+}
+
+sub setup_home_and_xdg_dirs {
+	my $home_dir = generated_file('_source', 'home', 0);
+	my $xdg_rundir = generated_file('_source', 'xdg-runtime-dir', 0);
+	my $creating_rundir = -d $xdg_rundir ? 0 : 1;
+	my @paths = (
+		$home_dir,
+		$xdg_rundir,
+	);
+	my @clear_env = qw(
+		XDG_CACHE_HOME
+		XDG_CONFIG_DIRS
+		XDG_CONFIG_HOME
+		XDG_DATA_HOME
+		XDG_DATA_DIRS
+	);
+	install_dir(@paths);
+	if ($creating_rundir) {
+		chmod(0700, $xdg_rundir) == 1 or warning("chmod(0700, \"$xdg_rundir\") failed: $! (ignoring)");
+	}
+	for my $envname (@clear_env) {
+		delete($ENV{$envname});
+	}
+	$ENV{'HOME'} = $home_dir;
+	$ENV{'XDG_RUNTIME_DIR'} = $xdg_rundir;
+	return;
+}
+
 # Sets environment variables from dpkg-buildflags. Avoids changing
 # any existing environment variables.
 sub set_buildflags {
@@ -2427,13 +2626,13 @@ sub open_gz {
 	};
 	if ($@) {
 		open($fd, '-|', 'gzip', '-dc', $file)
-		  or die("gzip -dc $file failed: $!");
+		  or error("gzip -dc $file failed: $!");
 	} else {
 		# Pass ":unix" as well due to https://rt.cpan.org/Public/Bug/Display.html?id=114557
 		# Alternatively, we could ensure we always use "POSIX::_exit".  Unfortunately,
 		# loading POSIX is insanely slow.
 		open($fd, '<:unix:gzip', $file)
-		  or die("open $file [<:unix:gzip] failed: $!");
+		  or error("open $file [<:unix:gzip] failed: $!");
 	}
 	return $fd;
 }
@@ -2655,17 +2854,54 @@ sub dbgsym_tmpdir {
 	return "debian/.debhelper/${package}/dbgsym-root";
 }
 
+sub perl_cross_incdir {
+	return if !is_cross_compiling();
+
+	# native builds don't currently need this so only load it on demand
+	require Config; Config->import();
+
+	my $triplet = dpkg_architecture_value("DEB_HOST_MULTIARCH");
+	my $perl_version = $Config::Config{version};
+	my $incdir = "/usr/lib/$triplet/perl/cross-config-${perl_version}";
+	return undef if !-e "$incdir/Config.pm";
+	return $incdir;
+}
 
 {
 	my %known_packages;
+	sub is_known_package {
+		my ($package) = @_;
+		%known_packages = map { $_ => 1 } getpackages() if not %known_packages;
+		return 1 if exists($known_packages{$package});
+		return 0
+	}
+
 	sub assert_opt_is_known_package {
 		my ($package, $method) = @_;
-		%known_packages = map { $_ => 1 } getpackages() if not %known_packages;
-		if (not exists($known_packages{$package})) {
+		if (not is_known_package($package)) {
 			error("Requested unknown package $package via $method, expected one of: " . join(' ', getpackages()));
 		}
 		return 1;
 	}
 }
 
+{
+	my $_disable_file_seccomp;
+	sub _internal_optional_file_args {
+		if (not defined($_disable_file_seccomp)) {
+			my $consider_disabling_seccomp = 0;
+			if ($ENV{'FAKEROOTKEY'} or ($ENV{'LD_PRELOAD'}//'') =~ m/fakeroot/) {
+				$consider_disabling_seccomp = 1;
+			}
+			if ($consider_disabling_seccomp) {
+				my $has_no_sandbox = (qx_cmd('file', '--help') // '') =~ m/--no-sandbox/;
+				$consider_disabling_seccomp = 0 if not $has_no_sandbox;
+			}
+			$_disable_file_seccomp = $consider_disabling_seccomp;
+		}
+		return ('--no-sandbox') if $_disable_file_seccomp;
+		return;
+	}
+}
+
 1
